"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from paygentic_sdk.types import BaseModel, UNSET_SENTINEL
from paygentic_sdk.utils import FieldMetadata, PathParamMetadata, RequestMetadata
from pydantic import model_serializer
from typing import Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class RefundUsageEventRequestBodyTypedDict(TypedDict):
    refunded: bool
    r"""Set to true to mark the usage event as refunded. Once refunded, the event cannot be un-refunded."""
    reason: NotRequired[str]
    r"""Optional reason for the refund. Sample values: 'Customer request', 'Billing error', 'Service credit', 'System error correction'"""


class RefundUsageEventRequestBody(BaseModel):
    refunded: bool
    r"""Set to true to mark the usage event as refunded. Once refunded, the event cannot be un-refunded."""

    reason: Optional[str] = None
    r"""Optional reason for the refund. Sample values: 'Customer request', 'Billing error', 'Service credit', 'System error correction'"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["reason"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


class RefundUsageEventRequestTypedDict(TypedDict):
    id: str
    request_body: RefundUsageEventRequestBodyTypedDict


class RefundUsageEventRequest(BaseModel):
    id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]

    request_body: Annotated[
        RefundUsageEventRequestBody,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]
