"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from dataclasses import dataclass, field
import httpx
from paygentic_sdk.errors import PaygenticError
from paygentic_sdk.types import BaseModel
from typing import Any, Dict, Optional


class ErrorData(BaseModel):
    message: str
    r"""Human-readable error message"""
    error: Optional[str] = None
    r"""Error type or code"""
    details: Optional[Dict[str, Any]] = None
    r"""Additional error details"""


@dataclass(unsafe_hash=True)
class Error(PaygenticError):
    data: ErrorData = field(hash=False)

    def __init__(
        self, data: ErrorData, raw_response: httpx.Response, body: Optional[str] = None
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)
