"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .revenuestats import RevenueStats, RevenueStatsTypedDict
from datetime import datetime
from paygentic_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Dict, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class RevenueTimeBucketTypedDict(TypedDict):
    timestamp: datetime
    r"""Start time of this bucket"""
    subscription_stats: NotRequired[Dict[str, RevenueStatsTypedDict]]
    r"""Revenue stats per subscription ID. May include 'other' for subscriptions outside top N."""
    total_stats: NotRequired[RevenueStatsTypedDict]


class RevenueTimeBucket(BaseModel):
    timestamp: datetime
    r"""Start time of this bucket"""

    subscription_stats: Annotated[
        Optional[Dict[str, RevenueStats]], pydantic.Field(alias="subscriptionStats")
    ] = None
    r"""Revenue stats per subscription ID. May include 'other' for subscriptions outside top N."""

    total_stats: Annotated[
        Optional[RevenueStats], pydantic.Field(alias="totalStats")
    ] = None

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["subscriptionStats", "totalStats"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    RevenueTimeBucket.model_rebuild()
except NameError:
    pass
