"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from .createusageeventrequest import (
    CreateUsageEventRequest,
    CreateUsageEventRequestTypedDict,
)
from paygentic_sdk.types import BaseModel
from typing import List
from typing_extensions import TypedDict


class BatchCreateUsageEventsRequestTypedDict(TypedDict):
    events: List[CreateUsageEventRequestTypedDict]


class BatchCreateUsageEventsRequest(BaseModel):
    events: List[CreateUsageEventRequest]
