"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from paygentic_sdk.types import BaseModel
from paygentic_sdk.utils import FieldMetadata, PathParamMetadata, RequestMetadata
from typing_extensions import Annotated, TypedDict


class TerminateSubscriptionRequestBodyTypedDict(TypedDict):
    reason: str
    r"""Cancellation explanation text. Sample values: 'Customer requested cancellation', 'Payment failure', 'Service migration', 'Contract expiration'"""


class TerminateSubscriptionRequestBody(BaseModel):
    reason: str
    r"""Cancellation explanation text. Sample values: 'Customer requested cancellation', 'Payment failure', 'Service migration', 'Contract expiration'"""


class TerminateSubscriptionRequestTypedDict(TypedDict):
    id: str
    r"""The subscription ID"""
    request_body: TerminateSubscriptionRequestBodyTypedDict


class TerminateSubscriptionRequest(BaseModel):
    id: Annotated[
        str, FieldMetadata(path=PathParamMetadata(style="simple", explode=False))
    ]
    r"""The subscription ID"""

    request_body: Annotated[
        TerminateSubscriptionRequestBody,
        FieldMetadata(request=RequestMetadata(media_type="application/json")),
    ]
