"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from .paygenticerror import PaygenticError
from typing import Any, TYPE_CHECKING

from paygentic_sdk.utils.dynamic_imports import lazy_getattr, lazy_dir

if TYPE_CHECKING:
    from .badrequest import BadRequest, BadRequestUnion
    from .deletecustomerop import (
        DeleteCustomerConflictError,
        DeleteCustomerConflictErrorData,
    )
    from .deletefeeop import DeleteFeeConflictError, DeleteFeeConflictErrorData
    from .error import Error, ErrorData
    from .no_response_error import NoResponseError
    from .paygenticdefaulterror import PaygenticDefaultError
    from .responsevalidationerror import ResponseValidationError
    from .validationerror import ValidationError, ValidationErrorData

__all__ = [
    "BadRequest",
    "BadRequestUnion",
    "DeleteCustomerConflictError",
    "DeleteCustomerConflictErrorData",
    "DeleteFeeConflictError",
    "DeleteFeeConflictErrorData",
    "Error",
    "ErrorData",
    "NoResponseError",
    "PaygenticDefaultError",
    "PaygenticError",
    "ResponseValidationError",
    "ValidationError",
    "ValidationErrorData",
]

_dynamic_imports: dict[str, str] = {
    "BadRequest": ".badrequest",
    "BadRequestUnion": ".badrequest",
    "DeleteCustomerConflictError": ".deletecustomerop",
    "DeleteCustomerConflictErrorData": ".deletecustomerop",
    "DeleteFeeConflictError": ".deletefeeop",
    "DeleteFeeConflictErrorData": ".deletefeeop",
    "Error": ".error",
    "ErrorData": ".error",
    "NoResponseError": ".no_response_error",
    "PaygenticDefaultError": ".paygenticdefaulterror",
    "ResponseValidationError": ".responsevalidationerror",
    "ValidationError": ".validationerror",
    "ValidationErrorData": ".validationerror",
}


def __getattr__(attr_name: str) -> Any:
    return lazy_getattr(
        attr_name, package=__package__, dynamic_imports=_dynamic_imports
    )


def __dir__():
    return lazy_dir(dynamic_imports=_dynamic_imports)
