"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from dataclasses import dataclass, field
import httpx
from paygentic_sdk.errors import PaygenticError
from paygentic_sdk.models import validationerror as models_validationerror
from paygentic_sdk.types import BaseModel
from typing import List, Optional


class ValidationErrorData(BaseModel):
    message: str
    r"""Human-readable error message"""
    errors: List[models_validationerror.Error]
    r"""Array of field-specific validation errors"""
    error: Optional[models_validationerror.ErrorEnum] = "validation_error"
    r"""Error type indicating validation failure"""


@dataclass(unsafe_hash=True)
class ValidationError(PaygenticError):
    data: ValidationErrorData = field(hash=False)

    def __init__(
        self,
        data: ValidationErrorData,
        raw_response: httpx.Response,
        body: Optional[str] = None,
    ):
        fallback = body or raw_response.text
        message = str(data.message) or fallback
        super().__init__(message, raw_response, body)
        object.__setattr__(self, "data", data)
