"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from paygentic_sdk.types import BaseModel, UNSET_SENTINEL
import pydantic
from pydantic import model_serializer
from typing import Any, Dict, Optional
from typing_extensions import Annotated, NotRequired, TypedDict


class PriceFeatureInputTypedDict(TypedDict):
    feature_id: str
    r"""The feature to associate with this price"""
    entitlement_template: NotRequired[Dict[str, Any]]
    r"""Template for entitlement values when this feature is provisioned"""


class PriceFeatureInput(BaseModel):
    feature_id: Annotated[str, pydantic.Field(alias="featureId")]
    r"""The feature to associate with this price"""

    entitlement_template: Annotated[
        Optional[Dict[str, Any]], pydantic.Field(alias="entitlementTemplate")
    ] = None
    r"""Template for entitlement values when this feature is provisioned"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["entitlementTemplate"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m


try:
    PriceFeatureInput.model_rebuild()
except NameError:
    pass
