"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from paygentic_sdk.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class OffsetPaginationTypedDict(TypedDict):
    r"""Offset-based pagination response."""

    limit: NotRequired[int]
    r"""Number of items returned in the current page."""
    offset: NotRequired[int]
    r"""Number of items skipped."""
    total: NotRequired[int]
    r"""Total number of items available."""


class OffsetPagination(BaseModel):
    r"""Offset-based pagination response."""

    limit: Optional[int] = None
    r"""Number of items returned in the current page."""

    offset: Optional[int] = None
    r"""Number of items skipped."""

    total: Optional[int] = None
    r"""Total number of items available."""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["limit", "offset", "total"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m
