"""Code generated by Speakeasy (https://speakeasy.com). DO NOT EDIT."""

from __future__ import annotations
from paygentic_sdk.types import BaseModel, UNSET_SENTINEL
from pydantic import model_serializer
from typing import Optional
from typing_extensions import NotRequired, TypedDict


class ValidTaxAddressTypedDict(TypedDict):
    r"""Indicates whether the consumer address is valid for tax calculation when using Paygentic Tax. If valid=false, tax calculation will be skipped and internal invoice flow with default tax rate will be used."""

    valid: bool
    r"""Whether the consumer address is valid for tax calculation when using Paygentic Tax"""
    message: NotRequired[str]
    r"""Validation message if address is invalid. Sample values: 'Country is required for tax calculation', 'Invalid postal code format', 'State/province required for this country'"""


class ValidTaxAddress(BaseModel):
    r"""Indicates whether the consumer address is valid for tax calculation when using Paygentic Tax. If valid=false, tax calculation will be skipped and internal invoice flow with default tax rate will be used."""

    valid: bool
    r"""Whether the consumer address is valid for tax calculation when using Paygentic Tax"""

    message: Optional[str] = None
    r"""Validation message if address is invalid. Sample values: 'Country is required for tax calculation', 'Invalid postal code format', 'State/province required for this country'"""

    @model_serializer(mode="wrap")
    def serialize_model(self, handler):
        optional_fields = set(["message"])
        serialized = handler(self)
        m = {}

        for n, f in type(self).model_fields.items():
            k = f.alias or n
            val = serialized.get(k)

            if val != UNSET_SENTINEL:
                if val is not None or k not in optional_fields:
                    m[k] = val

        return m
